#ifndef __XCOLLIDER_H__
#define __XCOLLIDER_H__

#include "iengine/mesh.h"
#include "csgeom/box.h"

// class iCollider;
class xCoord;
struct iCollideSystem;
/**
 * This is the type for the collision callback
 * class xCollider; 
 * typedef void(*TCollisionCallback)(xCollider*, xCollider*);
 */

/**
 * needs an abstract interface  for the object!!
 */
struct xObject {
  virtual bool isCollidingWith(iCollideSystem* cds, xObject* pO) = 0;
  // virtual iCollider* getCollider() = 0;
  virtual iMeshWrapper* getMesh() = 0;
};


/**
 * Implements the collider used by ECODS
 * 
 */

class xCollider {
  /**
   * the object associated with this collider
   */
  xObject* pxObj;
  
  /**
   * Status information
   */
  bool bMeshMoved;
  bool bMeshChanged;
  
  /** temp
   * MUST USE iMeshWrapper->GetWorldBoundingBox instead
   * and use the csBox to get the coordinates
   *
   */
  csBox3 AABB;


  /**
   * needs this to update AABB on movememnt
   */
 public:
  xCoord* xAABB[6];
  
  /**
   * Gets the AABB coordinates
   */
  double OxLeft() { return AABB.MinX();};
  double OxRight() { return AABB.MaxX();};
  double OyLeft() { return AABB.MinY();};
  double OyRight() { return AABB.MaxY();};
  double OzLeft() { return AABB.MinZ();};
  double OzRight(){ return AABB.MaxZ();};
  
  /**
   * get/set status
   */
  bool Moved() { return bMeshMoved; };
  void setMovedFlag(bool bf);
  xObject* getObject() { return pxObj; };

  /**
   * potential collision with object ID n
   */
  bool isPotentiallyColliding(int id);

  /**
   * the potentially colliding objects with this object (PCO) sets
   * as resulted from ordering the AABB projections
   */
  short* PCO;
  
  /**
   * We need to construct this PCO and destroy it in the destructor
   */
  void buildPCO(int npco);
  
  /** phoney constructor
   */
  xCollider(xObject* pObj);

  ~xCollider();
  

  /**
   * helper
   */
  void printPCO(int npco);
};

#endif
